package br.usp.ime.pictware.application.album.service;

import java.util.Collection;
import java.util.List;
import java.util.logging.Logger;

import br.usp.ime.pictware.application.assinante.service.ManterAssinanteService;
import br.usp.ime.pictware.business.album.bo.Album;
import br.usp.ime.pictware.business.album.bo.CapaProvisoria;
import br.usp.ime.pictware.business.album.bo.Foto;
import br.usp.ime.pictware.business.album.dao.AlbumDAO;
import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.business.util.dao.DAOFactory;
import br.usp.ime.pictware.business.util.dao.exception.DAOException;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.CriterioBuscaAssinante;

public class ManterAlbumService {
	
	private static ManterAlbumService instance = new ManterAlbumService();
	
	private ManterAlbumService(){
		
	}
	
	public static ManterAlbumService getInstance(){
		return instance;
	}
	
	private AlbumDAO getAlbumDAO(){
		try {
			return DAOFactory.getInstance().getDao(AlbumDAO.class);
		} catch (DAOException daoException) {
			daoException.printStackTrace();
			return null;
		}
	}
	
	public void incluirAlbum(Assinante owner, String tituloAlbum, String descricaoAlbum){
		Album album = new Album();
		album.setTitulo(tituloAlbum);
		album.setDescricao(descricaoAlbum);
		album.setNumeroAcessos(0L);
		
		owner.addAlbum(album);
		getAlbumDAO().incluirAlbum(owner, album);
		album.setCapa(new CapaProvisoria());		
	}
	
	public void incluirFoto(Album album, byte[] bytesArquivo, String nomeArquivo, Long tamanhoArquivo, String tipoArquivo, String legendaFoto){
		Foto foto = new Foto();
		foto.setBytesArquivo(bytesArquivo);
		foto.setNomeArquivo(nomeArquivo);
		foto.setTamanhoArquivo(tamanhoArquivo);
		foto.setTipoArquivo(tipoArquivo);
		foto.setLegenda(legendaFoto);		
		
		foto.setNumeroAcessos(0L);
		
		album.addFoto(foto);
		getAlbumDAO().incluirFoto(album, foto);
	}
	
	public Collection<Album> buscarAlbuns(String login){
		List<Album> listaAlbuns = null;
		Assinante assinante = ManterAssinanteService.getInstance().buscarAssinante(new CriterioBuscaAssinante(login));
		if (assinante != null){
			listaAlbuns = assinante.getListaAlbuns();
		}
		return listaAlbuns;
	}
	
	public void editarAlbum(Album album, String titulo, String descricao){
		album.setTitulo(titulo);
		album.setDescricao(descricao);
		getAlbumDAO().atualizarAlbum(album);
	}
	
	public void excluirAlbum(Assinante owner, Album album){
		owner.getListaAlbuns().remove(album);
		getAlbumDAO().excluirAlbum(album);
	}
	
	public void excluirFoto(Album album, Foto foto){
		album.getListaFotos().remove(foto);
		getAlbumDAO().excluirFoto(foto);
	}
	
	public void visualizarAlbum(Album album){
		album.getListaFotos().clear();
		//Long numeroAcessos = 
		getAlbumDAO().atualizarNumeroAcessoAlbum(album);
		ObservadorNumeroAcessoAlbum.getInstance().awake();
		//album.setNumeroAcessos(numeroAcessos);
		getAlbumDAO().carregarFotos(album);		
	}
	
	public void visualizarFoto(Foto foto){
		//foto.foiAcessada();
		getAlbumDAO().atualizarNumeroAcessoFoto(foto);
	}

	public Long obterNumAcessosAlbum(Long idAlbum) {
		// TODO Auto-generated method stub
		return getAlbumDAO().obterNumeroAcessos(idAlbum);
	}
	
}
