package br.usp.ime.pictware.application.assinante.service;

import java.util.List;

import br.usp.ime.pictware.business.album.bo.Album;
import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.business.assinante.dao.AssinanteDAO;
import br.usp.ime.pictware.business.util.dao.DAOFactory;
import br.usp.ime.pictware.business.util.dao.exception.DAOException;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.CriterioBuscaAssinante;

public class ManterAssinanteService {

	private static ManterAssinanteService instance = new ManterAssinanteService();
	
	private ManterAssinanteService(){
		
	}

	public static ManterAssinanteService getInstance(){
		return instance;
	}
	
	public AssinanteDAO getAssinanteDAO(){
		try {
			return DAOFactory.getInstance().getDao(AssinanteDAO.class);
		} catch (DAOException daoException) {
			daoException.printStackTrace();
			return null;
		}
	}
	
	public Assinante buscarAssinante(Long userId){
		CriterioBuscaAssinante criterioBusca = new CriterioBuscaAssinante(userId);
		return buscarAssinante(criterioBusca);
	}
	
	public Assinante buscarAssinante(String login, String senha){
		CriterioBuscaAssinante criterioBusca = new CriterioBuscaAssinante(login,senha);
		return buscarAssinante(criterioBusca);
	}

	public Assinante buscarAssinante(String login){
		CriterioBuscaAssinante criterioBusca = new CriterioBuscaAssinante(login);
		return buscarAssinante(criterioBusca);
	}
	
	public Assinante buscarAssinante(CriterioBuscaAssinante criterioBusca){
		Assinante assinante = getAssinanteDAO().buscarAssinante(criterioBusca);
		if (assinante != null){
			List<Album> listaAlbuns = assinante.getListaAlbuns();
			for (Album album : listaAlbuns){
				if (!album.hasCapa()){
					album.colocarCapaProvisoria();
				}
			}
		}
		
		return assinante;
	}
	
	public void atualizarInfoAssinante(Assinante assinante, String senha, String nomeAssinante,
			String apelido) {
		assinante.setNome(nomeAssinante);
		assinante.setApelido(apelido);
		getAssinanteDAO().atualizarAssinante(assinante,senha);
	}

	public Assinante cadastrarAssinante(String login, String senha,
			String nomeAssinante, String apelido) {
		Assinante assinante = new Assinante();
		assinante.setLogin(login);
		assinante.setNome(nomeAssinante);
		assinante.setApelido(apelido);
		getAssinanteDAO().incluirAssinante(assinante, senha);
		return assinante;
	}
	
}
