package br.usp.ime.pictware.application.grupoacesso.service;


import br.usp.ime.pictware.application.assinante.service.ManterAssinanteService;
import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.business.grupoacesso.bo.GrupoAcesso;
import br.usp.ime.pictware.business.grupoacesso.dao.GrupoAcessoDAO;
import br.usp.ime.pictware.business.util.dao.DAOFactory;
import br.usp.ime.pictware.business.util.dao.exception.DAOException;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.CriterioBuscaAssinante;

public class ManterGrupoAcessoService {

	private static ManterGrupoAcessoService instance = new ManterGrupoAcessoService();
	
	private ManterGrupoAcessoService(){
		
	}
	
	public static ManterGrupoAcessoService getInstance(){
		return instance;
	}
	
	private GrupoAcessoDAO getGrupoAcessoDAO(){
		try {
			return DAOFactory.getInstance().getDao(GrupoAcessoDAO.class);
		} catch (DAOException daoException) {
			daoException.printStackTrace();
			return null;
		}
	}
	
	public Long incluirGrupoAcesso(Long ownerId, String nomeGrupo){
		
		GrupoAcesso grupoAcesso = new GrupoAcesso();
		grupoAcesso.setNome(nomeGrupo);
		
		GrupoAcessoDAO grupoAcessoDAO = getGrupoAcessoDAO();
		boolean existeGrupoMesmoNome = grupoAcessoDAO.existeGrupo(ownerId, grupoAcesso);
		if (!existeGrupoMesmoNome){
			
			//TODO: Utilizar estratgia abaixo quando implementarmos
			//um framework de persistncia
			
			//Assinante owner = ManterAssinanteService.getInstance().buscarAssinante(ownerId);
			//owner.addGrupoAcesso(grupoAcesso);
			//ManterAssinanteService.getInstance().salvarAssinante()
			
			grupoAcessoDAO.incluirGrupoAcesso(ownerId, grupoAcesso);
			return grupoAcesso.getId();
		}
		else{
			//TODO: tratar grupo acesso duplicado
			System.out.println("Grupo de acesso duplicado");
			return null;
		}
	}
	
	public void excluirGrupoAcesso(Long ownerId, Long idGrupoAcesso){
		
		//TODO: Utilizar estratgia abaixo quando implementarmos
		//um framework de persistncia
		
		//Assinante owner = ManterAssinanteService.getInstance().buscarAssinante(ownerId);
		//owner.removeGrupoAcesso(grupoAcesso);
		//ManterAssinanteService.getInstance().salvarAssinante()
		GrupoAcessoDAO grupoAcessoDAO = getGrupoAcessoDAO();
		grupoAcessoDAO.excluirGrupoAcesso(idGrupoAcesso);
	}
	
	public void incluirParticipante(GrupoAcesso grupoAcesso, String login){
		CriterioBuscaAssinante criterioBuscaAssinante = new CriterioBuscaAssinante(login);
		Assinante membro = ManterAssinanteService.getInstance().buscarAssinante(criterioBuscaAssinante);
		
		//Verifica se a busca retornou um assinante
		if (membro != null){
			GrupoAcessoDAO grupoAcessoDAO = getGrupoAcessoDAO();
			grupoAcessoDAO.incluirMembro(grupoAcesso, membro);
		}
	}
	
}
