package br.usp.ime.pictware.infraestrutura.persistencia.album;

import java.io.ByteArrayInputStream;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;

import br.usp.ime.pictware.business.album.bo.Album;
import br.usp.ime.pictware.business.album.bo.Foto;
import br.usp.ime.pictware.business.album.dao.AlbumDAO;
import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.infraestrutura.persistencia.util.ConnectionManager;

public class AlbumDAOJDBC implements AlbumDAO{
	
	private static final Logger LOG = Logger.getLogger(AlbumDAOJDBC.class.getName());
	
	private String incluirAlbum = "INSERT INTO ALBUM (TITULO, DESCRICAO, NUMEROACESSO, IDOWNER) " +
								  "VALUES (?,?,?,?)";
	
	private String incluirFoto = "INSERT INTO FOTO (LEGENDA, NUMEROACESSO, DADOS, IDALBUM, NOMEARQUIVO, TAMANHOARQUIVO, TIPOARQUIVO) " +
	  "VALUES (?,?,?,?,?,?,?)";
	
	private String atualizarAlbum = "UPDATE ALBUM SET TITULO = ?, DESCRICAO = ? WHERE ID = ?";
	
	private String carregarFotosAlbum = "SELECT * FROM FOTO AS FOTO WHERE FOTO.IDALBUM = ?";
	
	private String excluirAlbum = "DELETE FROM ALBUM WHERE ALBUM.ID = ?";
	
	private String excluirFoto = "DELETE FROM FOTO WHERE FOTO.ID = ?";
	
	private String atualizarNumeroAcessosAlbum = "UPDATE ALBUM SET NUMEROACESSO = NUMEROACESSO + 1 WHERE ID = ?";

	private String atualizarNumeroAcessosFoto = "UPDATE FOTO SET NUMEROACESSO = NUMEROACESSO + 1 WHERE ID = ?";
	
	private String obterNumeroAcessoAlbum = "SELECT ALBUM.NUMEROACESSO FROM ALBUM WHERE ALBUM.ID = ?"; 
	
	public void incluirAlbum(Assinante owner, Album album) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(incluirAlbum,Statement.RETURN_GENERATED_KEYS);
			st.setString(1, album.getTitulo());
			st.setString(2, album.getDescricao());
			st.setLong(3, album.getNumeroAcessos());
			st.setLong(4, owner.getId());
			st.executeUpdate();
			ResultSet rs = st.getGeneratedKeys();
			rs.next();
			album.setId(rs.getLong(1));
			
			//TODO: Grupo Acesso
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao incluir Album no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
	}
	
	public void incluirFoto(Album album, Foto foto) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(incluirFoto,Statement.RETURN_GENERATED_KEYS);
			
			st.setString(1, foto.getLegenda());
			st.setLong(2, foto.getNumeroAcessos());
			st.setBinaryStream(3, new ByteArrayInputStream(foto.getBytesArquivo()), foto.getBytesArquivo().length);
			st.setLong(4, album.getId());
			st.setString(5, foto.getNomeArquivo());
			st.setLong(6, foto.getTamanhoArquivo());
			st.setString(7, foto.getTipoArquivo());
			st.executeUpdate();
			ResultSet rs = st.getGeneratedKeys();
			rs.next();
			foto.setId(rs.getLong(1));
			
			//TODO: Grupo Acesso			
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao incluir Foto no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}		
	}

	public void excluirAlbum(Album album) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(excluirAlbum);
			st.setLong(1, album.getId());
			st.execute();
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao excluir Album do repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
		
	}

	public void excluirFoto(Foto foto) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(excluirFoto);
			st.setLong(1, foto.getId());
			st.execute();	
		
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao excluir Foto do repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}		
	}
	
	public void atualizarAlbum(Album album){
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(atualizarAlbum);
			st.setString(1, album.getTitulo());
			st.setString(2, album.getDescricao());
			st.setLong(3, album.getId());
			st.executeUpdate();
			
			//TODO: Grupo Acesso			
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao atualizar Album no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
	}

	public void carregarFotos(Album album) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(carregarFotosAlbum);
			st.setLong(1, album.getId());
			
			ResultSet rs = st.executeQuery();
			while (rs.next()){
				Foto foto = new Foto();
				foto.setId(rs.getLong("ID"));
				foto.setLegenda(rs.getString("LEGENDA"));
				foto.setNumeroAcessos(rs.getLong("NUMEROACESSO"));
				foto.setNomeArquivo(rs.getString("NOMEARQUIVO"));
				foto.setTamanhoArquivo(rs.getLong("TAMANHOARQUIVO"));
				foto.setTipoArquivo(rs.getString("TIPOARQUIVO"));
				
				Blob blob = rs.getBlob("DADOS");
				byte[] bytes = blob.getBytes(1, (int)blob.length());
				foto.setBytesArquivo(bytes);
				
				album.addFoto(foto);
			}
			
			conn.close();
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao carregar as Fotos do Album", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}	
	}		


	public void atualizarNumeroAcessoAlbum(Album album) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(atualizarNumeroAcessosAlbum);
			st.setLong(1, album.getId());
			st.executeUpdate();
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao atualizar Numero de Acessos do Album no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
		
	}

	public void atualizarNumeroAcessoFoto(Foto foto) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(atualizarNumeroAcessosFoto);
			st.setLong(1, foto.getId());
			st.executeUpdate();			
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao atualizar Numero de Acessos da Foto no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
		
	}

	public Long obterNumeroAcessos(Long idAlbum) {
		
		Long numeroAcessos = -1L;
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(obterNumeroAcessoAlbum);
			st.setLong(1, idAlbum);
			ResultSet rs = st.executeQuery();
			if (rs.next()){
				numeroAcessos = rs.getLong("NUMEROACESSO"); 
			}			
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao atualizar Numero de Acessos da Foto no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
		
		return numeroAcessos;
	}
}