package br.usp.ime.pictware.infraestrutura.persistencia.assinante;

import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;

import br.usp.ime.pictware.business.album.bo.Album;
import br.usp.ime.pictware.business.album.bo.Foto;
import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.business.assinante.dao.AssinanteDAO;
import br.usp.ime.pictware.business.grupoacesso.bo.GrupoAcesso;
import br.usp.ime.pictware.infraestrutura.persistencia.util.ConnectionManager;

public class AssinanteDAOJDBC implements AssinanteDAO{
	
	private static final Logger LOG = Logger.getLogger(AssinanteDAOJDBC.class.getName());	
	
	private String findAlbunsComCapaAssinante = "SELECT ALBUM.ID AS IDALBUM, ALBUM.TITULO AS TITULOALBUM, " +
	"ALBUM.DESCRICAO AS DESCRICAOALBUM, ALBUM.NUMEROACESSO AS NUMEROACESSOALBUM, " +
	"FOTO.ID AS IDFOTO, FOTO.LEGENDA AS LEGENDAFOTO, FOTO.NUMEROACESSO AS NUMEROACESSOFOTO, " +
	"FOTO.DADOS AS DADOSFOTO FROM ALBUM, FOTO WHERE " +
	"ALBUM.IDOWNER = ? AND FOTO.ID = ALBUM.IDCAPA";
	
	private String findAlbunsSemCapaAssinante = "SELECT * FROM ALBUM WHERE ALBUM.IDCAPA IS NULL AND " +
	"ALBUM.IDOWNER = ?";
	
	private String findGruposAcesso = "SELECT * FROM GRUPOACESSO WHERE IDOWNER = ?";
	
	private String atualizarInfoAssinante = "UPDATE ASSINANTE SET SENHA = ? WHERE ID = ?";
	private String atualizarInfoPessoa = "UPDATE PESSOA SET NOME = ?, APELIDO = ? WHERE ID IN " +
	"(SELECT ASSINANTE.IDPESSOA FROM ASSINANTE WHERE ASSINANTE.ID = ?)";
	
	private String inserirInfoPessoa = "INSERT INTO PESSOA (NOME, APELIDO) VALUES (?,?)";
	private String inserirInfoAssinante = "INSERT INTO ASSINANTE (LOGIN, SENHA, IDPESSOA) VALUES (?,?,?)";
		
	public Assinante buscarAssinante(CriterioBuscaAssinante criterioBuscaAssinante) {
			 			
		Assinante assinante = null;
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(criterioBuscaAssinante.getQuery());
			ResultSet rs = st.executeQuery();
			
			if (rs.next()){				
				assinante = new Assinante();
				assinante.setId(rs.getLong("IDASSINANTE"));
				assinante.setNome(rs.getString("NOME"));
				assinante.setApelido(rs.getString("APELIDO"));
				assinante.setLogin(rs.getString("LOGIN"));
				
				//Fecha o resultSet
				rs.close();
				
				//Resgata os Albuns c/ Capa
				st = conn.prepareStatement(findAlbunsComCapaAssinante);
				st.setLong(1, assinante.getId());
				
				rs = st.executeQuery();
				
				while (rs.next()){
					Album album = new Album();
					album.setId(rs.getLong("IDALBUM"));
					album.setTitulo(rs.getString("TITULO"));
					album.setDescricao(rs.getString("DESCRICAOALBUM"));
					album.setNumeroAcessos(rs.getLong("NUMEROACESSOALBUM"));
					album.setIdOwner(assinante.getId());
										
					Foto capa = new Foto();
					capa.setId(rs.getLong("IDFOTO"));
					capa.setLegenda(rs.getString("LEGENDAFOTO"));
					capa.setNumeroAcessos(rs.getLong("NUMEROACESSOFOTO"));
					
					Blob blob = rs.getBlob("DADOSFOTO");
					byte[] bytes = blob.getBytes(1, (int)blob.length());
					capa.setBytesArquivo(bytes);
					
					album.setCapa(capa);					
					assinante.addAlbum(album);
				}
				
				rs.close();
				
				//Resgata os Albuns sem Capa
				st = conn.prepareStatement(findAlbunsSemCapaAssinante);
				st.setLong(1, assinante.getId());
				rs = st.executeQuery();
				
				while (rs.next()){
					Album album = new Album();
					album.setId(rs.getLong("ID"));
					album.setTitulo(rs.getString("TITULO"));
					album.setDescricao(rs.getString("DESCRICAO"));
					album.setNumeroAcessos(rs.getLong("NUMEROACESSO"));
					album.setIdOwner(assinante.getId());
								
					assinante.addAlbum(album);
				}
				
				//Resgata os Grupos de Acesso
				st = conn.prepareStatement(findGruposAcesso);
				st.setLong(1, assinante.getId());
				rs = st.executeQuery();
				
				while (rs.next()){
					GrupoAcesso grupoAcesso = new GrupoAcesso();
					grupoAcesso.setId(rs.getLong("ID"));
					grupoAcesso.setNome(rs.getString("NOME"));
					assinante.addGrupoAcesso(grupoAcesso);
				}
				
				rs.close();
				
				//TODO: Resgatar participantes do grupo de acesso								
			}			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao buscar Assinante no repositrio", e);
			e.printStackTrace();			
		}
		finally{
			ConnectionManager.getInstance().closeConnection(conn);			
		}
		
		return assinante;
	}
	
	public void atualizarAssinante(Assinante assinante, String senha){
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(atualizarInfoAssinante);
			st.setString(1, senha);
			st.setLong(2, assinante.getId());
			st.executeUpdate();
			
			st = conn.prepareStatement(atualizarInfoPessoa);
			st.setString(1, assinante.getNome());
			st.setString(2, assinante.getApelido());
			st.setLong(3, assinante.getId());
			st.executeUpdate();
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao atualizar informaes do Assinante no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
	}

	public void incluirAssinante(Assinante assinante, String senha) {
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
						
			PreparedStatement st = conn.prepareStatement(inserirInfoPessoa,Statement.RETURN_GENERATED_KEYS);
			st.setString(1, assinante.getNome());
			st.setString(2, assinante.getApelido());
			st.executeUpdate();
			ResultSet rs = st.getGeneratedKeys();
			rs.next();
			Long idPessoa = rs.getLong(1);
			
			st = conn.prepareStatement(inserirInfoAssinante);
			
			st.setString(1, assinante.getLogin());
			st.setString(2, senha);
			st.setLong(3, idPessoa);
			st.executeUpdate();	
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao incluir Assinante no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
		
	}

}
