package br.usp.ime.pictware.infraestrutura.persistencia.grupoacesso;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;

import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.business.grupoacesso.bo.GrupoAcesso;
import br.usp.ime.pictware.business.grupoacesso.dao.GrupoAcessoDAO;
import br.usp.ime.pictware.infraestrutura.persistencia.util.ConnectionManager;

public class GrupoAcessoDAOJDBC implements GrupoAcessoDAO{

	private static final Logger LOG = Logger.getLogger(GrupoAcessoDAOJDBC.class.getName());	
	
	private String incluirGrupoAcesso = "INSERT INTO GRUPOACESSO (NOME, IDOWNER) VALUES (?,?)";
	private String incluirParticipante = "INSERT INTO GRUPOACESSO (IDGRUPO, IDASSINANTE) VALUES (?,?)";
	private String existeGrupoAcesso = "SELECT COUNT(*) FROM GRUPOACESSO WHERE IDOWNER = ? AND NOME = ?";
	private String removerGrupoAcesso = "DELETE FROM GRUPOACESSO WHERE ID = ?";
	
	public void incluirGrupoAcesso(Long ownerId, GrupoAcesso grupoAcesso) {
				
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(incluirGrupoAcesso,Statement.RETURN_GENERATED_KEYS);
			st.setString(1,grupoAcesso.getNome());
			st.setLong(2,ownerId);
			st.executeUpdate();
			ResultSet rs = st.getGeneratedKeys();
			rs.next();
			grupoAcesso.setId(rs.getLong(1));		
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao incluir Grupo de Acesso no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
	}

	public void excluirGrupoAcesso(Long idGrupoAcesso) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(removerGrupoAcesso);
			st.setLong(1,idGrupoAcesso);
			st.executeUpdate();		
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao remover Grupo de Acesso do repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
	}

	public void incluirMembro(GrupoAcesso grupoAcesso, Assinante participante) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(incluirParticipante);
			st.setLong(1,grupoAcesso.getId());
			st.setLong(2,participante.getId());
			st.executeUpdate();
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "Erro ao incluir Membro de Grupo no repositrio", e);
			e.printStackTrace();
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
	}

	public boolean existeGrupo(Long idOwner, GrupoAcesso grupoAcesso) {
		
		Connection conn = null;		
		try{
			conn = ConnectionManager.getInstance().openConnection();
			
			PreparedStatement st = conn.prepareStatement(existeGrupoAcesso);
			st.setLong(1,idOwner);
			st.setString(2,grupoAcesso.getNome());
			ResultSet rs = st.executeQuery();
			rs.next();
			Long count = rs.getLong(1);
			return (count > 0);
			
		}catch(Exception e){
			LOG.log(Level.SEVERE, "No foi possvel verificar existencia de grupo de acesso",e);
			e.printStackTrace();
			return false;
		}finally{
			ConnectionManager.getInstance().closeConnection(conn);
		}
	}
	
}
