package br.usp.ime.pictware.web.framework.controller;

import java.io.IOException;
import java.util.logging.Logger;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import br.usp.ime.pictware.web.framework.action.Action;
import br.usp.ime.pictware.web.framework.action.ActionForward;
import br.usp.ime.pictware.web.framework.action.ActionMapping;


/**
 * @author goliva
 * 
 * Front Controller J2EE Design Pattern Implementation 
 * using Command and Controller Strategy. Consequences:
 * 	 - Centralizes Control
 * 	 - Improves Manageability of Security
 *   - Improves Reusability 
 */
public class ServletFrontController extends HttpServlet {

	private static final long serialVersionUID = 2028520695648368844L;
	private static String LOG_NAME = ServletFrontController.class.getName();
	private static Logger LOGGER = Logger.getLogger(LOG_NAME);
	
	/**
     * @see HttpServlet#HttpServlet()
     */
    public ServletFrontController() {
        super();
        LOGGER.info("Loading ServletFrontController");
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException{
		doPost(request, response);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		LOGGER.entering(LOG_NAME,"doPost(HttpServletRequest request, HttpServletResponse response)");
		try{			
			RequestHelper requestHelper = new RequestHelper(getServletContext(),request);
			Action action = requestHelper.getAction();
			ActionMapping actionMapping = requestHelper.getActionMapping();
			ActionForward actionForward = action.execute(actionMapping, request, response);
			if (actionForward != null){
				RequestDispatcher view = request.getRequestDispatcher(actionForward.getJSPPage());
				view.forward(request, response);
			}
		}finally{
			LOGGER.exiting(LOG_NAME,"doPost(HttpServletRequest request, HttpServletResponse response)");	
		}	
	}
	
	

}
